<?php

namespace EscapeWork\Manager\Console;

use Illuminate\Support\Str;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Console\AppNamespaceDetectorTrait;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;

class CrudGeneratorTemplateCommand extends Command
{

    use AppNamespaceDetectorTrait;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'manager:crud:template {template?} {--with-migration}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cria um CRUD com base em um template pré definido (Opções: news)';

    /**
     * Available templates
     */
    protected $templates = [
        'news' => [
            'title'      => 'Notícias',
            'variable'   => 'new',
            'model'      => 'News',
            'route'      => 'noticias',
            'routeAlias' => 'news',
            'folder'     => 'news',
            'controller' => 'NewsController',
        ],

        'categories' => [
            'title'      => 'Categorias',
            'variable'   => 'category',
            'model'      => 'Category',
            'route'      => 'categories',
            'routeAlias' => 'categories',
            'folder'     => 'categories',
            'controller' => 'CategoriesController',
        ],

        'products' => [
            'title'      => 'Produtos',
            'variable'   => 'product',
            'model'      => 'Product',
            'route'      => 'products',
            'routeAlias' => 'products',
            'folder'     => 'products',
            'controller' => 'ProductsController',
        ],

        'banners' => [
            'title'      => 'Banners',
            'variable'   => 'banner',
            'model'      => 'Banner',
            'route'      => 'banners',
            'routeAlias' => 'banners',
            'folder'     => 'banners',
            'controller' => 'BannersController',
        ],
    ];

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function fire()
    {
        $templates = require(__DIR__ . '/../../config/templates.php');

        if (! $this->argument('template') || ! array_key_exists($this->argument('template'), $templates)) {
            $this->showOptions($templates);
            return;
        }

        $key       = $this->argument('template');
        $template  = $templates[$key];
        $namespace = $this->getAppNamespace();

        $this->info(' -> Gerando os arquivos para o ' . $template['description']);
        $this->line('');

        foreach ($template['files'] as $file => $data) {
            $path = base_path() . '/' . $data['path'];
            $stub = File::get(__DIR__ . '/../../resources/stubs/'.$key.'/' . $data['stub']);
            $stub = str_replace('{{namespace}}', $namespace, $stub);
            $file = str_replace('{{datetime}}', date('Y_m_d_His'), $file);

            if (! File::isDirectory($path)) {
                File::makeDirectory($path);
            }

            if (! File::exists($path . '/' . $file . $data['extension'])) {
                File::put($path . '/' . $file . $data['extension'], $stub);

                $this->comment(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' criado com sucesso!');
            } else {
                $this->error(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' já existe e não foi sobreescrito.');
            }
        }

        $this->line('');
        $this->info(' -> Cadastro criado com sucesso!');
        $this->line('');

        # rotas
        $this->info(' -> Não esqueça de adicionar as rotas abaixo no seu arquivo de rotas (<comment>routes.php</comment>)');
        $this->line("     Route::get('".$template['route']['url']."', ['as' => '".$template['route']['name'].".index', 'uses' => '".$template['route']['controller']."@index']);");
        $this->line("     Route::get('".$template['route']['url']."/create', ['as' => '".$template['route']['name'].".create', 'uses' => '".$template['route']['controller']."@create']);");
        $this->line("     Route::post('".$template['route']['url']."', ['as' => '".$template['route']['name'].".store', 'uses' => '".$template['route']['controller']."@store']);");
        $this->line("     Route::get('".$template['route']['url']."/{id}/edit', ['as' => '".$template['route']['name'].".edit', 'uses' => '".$template['route']['controller']."@edit']);");
        $this->line("     Route::put('".$template['route']['url']."/{id}', ['as' => '".$template['route']['name'].".update', 'uses' => '".$template['route']['controller']."@update']);");
        $this->line("     Route::put('".$template['route']['url']."/{id}/basic', ['as' => '".$template['route']['name'].".updateBasic', 'uses' => '".$template['route']['controller']."@updateBasic']);");
        $this->line("     Route::delete('".$template['route']['url']."/{id}', ['as' => '".$template['route']['name'].".destroy', 'uses' => '".$template['route']['controller']."@destroy']);");

        # adicionando no menu
        $html = '<li class="{{ $page == \''.$template['route']['name'].'\' ? \'active\' : null }}">
    <a href="{{ route(\'manager.'.$template['route']['name'].'.index\') }}">' . $template['title'] . '</a>
</li>';

        if (\File::exists($menu = base_path().'/resources/views/manager/menu.blade.php')) {
            \File::append($menu, "\n" . $html);

            $this->line('');
            $this->info(' -> Adicionamos o HTML com o link na view do menu (resources/views/manager/menu.blade.php)');
        } else {
            $this->line('');
            $this->comment(' -> E adicione no seu menu');
            $this->line($html);
        }

        if ($this->option('with-migration')) {
            $this->info(' -> E não esqueça de executar a migration!');
        }

        if (isset($template['obs'])) {
            $this->comment(' -> ' . $template['obs']);
        }
    }

    public function showOptions($templates)
    {
        $this->info('Os templates disponíveis são os seguintes: ');
        $this->line('');

        $headers = ['Template', 'Descrição', 'Uso'];
        $content = [];

        foreach ($templates as $type => $template) {
            $content[] = [
                $type, $template['description'], 'php artisan manager:crud:template ' . $type
            ];
        }

        $this->table($headers, $content);
    }
}
